@@ -20,6 +20,7 @@
 
 #include "MessageActionsModel.h"
 #include "resources/ResourcesModel.h"
+#include "utils.h"
 #include <QAction>
 
 MessageActionsModel::MessageActionsModel(QObject* parent)
@@ -48,20 +49,15 @@ int MessageActionsModel::rowCount(const QModelIndex& parent) const
 
 void MessageActionsModel::reload()
 {
-    const auto actions = ResourcesModel::global()->messageActions();
+    auto actions = ResourcesModel::global()->messageActions();
+    if (m_priority>=0) {
+        actions = kFilter<QList<QAction*>>(actions, [this](QAction* action){ return action->priority() == m_priority; });
+    }
     if (actions == m_actions)
         return;
 
     beginResetModel();
     m_actions = actions;
-    if (m_priority>=0) {
-        for(auto it=m_actions.begin(); it!=m_actions.end(); ) {
-            if ((*it)->priority() == m_priority) {
-                ++it;
-            } else
-                it = m_actions.erase(it);
-        }
-    }
     endResetModel();
 }
 
@@ -77,3 +73,8 @@ void MessageActionsModel::setFilterPriority(int p)
         reload();
     }
 }
+
+void MessageActionsModel::componentComplete()
+{
+    reload();
+}
