@@ -27,6 +27,7 @@
 #include <resources/AbstractResourcesBackend.h>
 #include <QVariantList>
 #include <QSharedPointer>
+#include <QThreadPool>
 
 #include <AppStreamQt/component.h>
 
@@ -54,12 +55,15 @@ public:
     bool isValid() const override;
     QList<QAction*> messageActions() const override { return {}; }
 
-    void installApplication(AbstractResource* app) override;
-    void installApplication(AbstractResource* app, const AddonList& addons) override;
-    void removeApplication(AbstractResource* app) override;
+    Transaction* installApplication(AbstractResource* app) override;
+    Transaction* installApplication(AbstractResource* app, const AddonList& addons) override;
+    Transaction* removeApplication(AbstractResource* app) override;
     bool isFetching() const override { return m_fetching; }
     AbstractResource * resourceForFile(const QUrl & ) override;
     void checkForUpdates() override;
+    QString displayName() const override;
+    bool hasApplications() const override { return true; }
+    FlatpakResource * addSourceFromFlatpakRepo(const QUrl &url);
 
 private Q_SLOTS:
     void onFetchMetadataFinished(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource, const QByteArray &metadata);
@@ -71,20 +75,20 @@ private:
     FlatpakInstallation * preferredInstallation() const { return m_installations.constFirst(); }
     void integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote);
     FlatpakRemote * getFlatpakRemoteByUrl(const QString &url, FlatpakInstallation *installation) const;
-    FlatpakInstalledRef * getInstalledRefForApp(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource);
-    FlatpakResource * getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref);
-    FlatpakResource * getRuntimeForApp(FlatpakResource *resource);
+    FlatpakInstalledRef * getInstalledRefForApp(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource) const;
+    FlatpakResource * getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref) const;
+    FlatpakResource * getRuntimeForApp(FlatpakResource *resource) const;
 
     FlatpakResource * addAppFromFlatpakBundle(const QUrl &url);
     FlatpakResource * addAppFromFlatpakRef(const QUrl &url);
-    FlatpakResource * addSourceFromFlatpakRepo(const QUrl &url);
     void addResource(FlatpakResource *resource);
-    bool compareAppFlatpakRef(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource, FlatpakInstalledRef *ref);
+    bool compareAppFlatpakRef(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource, FlatpakInstalledRef *ref) const;
     bool loadAppsFromAppstreamData(FlatpakInstallation *flatpakInstallation);
     bool loadInstalledApps(FlatpakInstallation *flatpakInstallation);
     void loadLocalUpdates(FlatpakInstallation *flatpakInstallation);
     void loadRemoteUpdates(FlatpakInstallation *flatpakInstallation);
     bool parseMetadataFromAppBundle(FlatpakResource *resource);
+    void refreshAppstreamMetadata(FlatpakInstallation *installation, FlatpakRemote *remote);
     void reloadPackageList();
     bool setupFlatpakInstallations(GError **error);
     void updateAppInstalledMetadata(FlatpakInstalledRef *installedRef, FlatpakResource *resource);
@@ -105,6 +109,7 @@ private:
 
     GCancellable *m_cancellable;
     QVector<FlatpakInstallation *> m_installations;
+    QThreadPool m_threadPool;
 };
 
 #endif // FLATPAKBACKEND_H
