@@ -40,7 +40,6 @@
 
 // DiscoverCommon includes
 #include "Transaction/Transaction.h"
-#include "Transaction/TransactionModel.h"
 #include "Category/Category.h"
 
 // Own includes
@@ -120,6 +119,7 @@ KNSBackend::KNSBackend(QObject* parent, const QString& iconName, const QString &
         Q_EMIT availableForQueries();
         this->setFetching(false);
         qWarning() << "kns error" << objectName() << error;
+        passiveMessage(i18n("%1: %2", name(), error));
     });
     connect(m_engine, &KNSCore::Engine::signalEntriesLoaded, this, &KNSBackend::receivedEntries, Qt::QueuedConnection);
     connect(m_engine, &KNSCore::Engine::signalEntryChanged, this, &KNSBackend::statusChanged);
@@ -248,8 +248,6 @@ public:
         : Transaction(parent, res, role)
         , m_id(res->entry().uniqueId())
     {
-        TransactionModel::global()->addTransaction(this);
-
         setCancellable(false);
 
         auto manager = res->knsBackend()->engine();
@@ -272,43 +270,36 @@ public:
                 case KNS3::Entry::Updateable:
                     if (status() != DoneStatus) {
                         setStatus(DoneStatus);
-                        TransactionModel::global()->removeTransaction(this);
                     }
                     break;
             }
         }
     }
 
-    ~KNSTransaction() override {
-        if (TransactionModel::global()->contains(this)) {
-            qWarning() << "deleting Transaction before it's done";
-                TransactionModel::global()->removeTransaction(this);
-        }
-    }
-
     void cancel() override {}
 
 private:
     const QString m_id;
 };
 
-void KNSBackend::removeApplication(AbstractResource* app)
+Transaction* KNSBackend::removeApplication(AbstractResource* app)
 {
     auto res = qobject_cast<KNSResource*>(app);
-    new KNSTransaction(this, res, Transaction::RemoveRole);
+    auto t = new KNSTransaction(this, res, Transaction::RemoveRole);
     m_engine->uninstall(res->entry());
+    return t;
 }
 
-void KNSBackend::installApplication(AbstractResource* app)
+Transaction* KNSBackend::installApplication(AbstractResource* app)
 {
     auto res = qobject_cast<KNSResource*>(app);
     m_engine->install(res->entry());
-    new KNSTransaction(this, res, Transaction::InstallRole);
+    return new KNSTransaction(this, res, Transaction::InstallRole);
 }
 
-void KNSBackend::installApplication(AbstractResource* app, const AddonList& /*addons*/)
+Transaction* KNSBackend::installApplication(AbstractResource* app, const AddonList& /*addons*/)
 {
-    installApplication(app);
+    return installApplication(app);
 }
 
 int KNSBackend::updatesCount() const
@@ -328,7 +319,12 @@ static ResultsStream* voidStream()
 
 ResultsStream* KNSBackend::search(const AbstractResourcesBackend::Filters& filter)
 {
-    if (filter.state >= AbstractResource::Installed) {
+    if ((!filter.resourceUrl.isEmpty() && filter.resourceUrl.scheme() != QLatin1String("kns")) || !filter.mimetype.isEmpty())
+        return voidStream();
+
+    if (filter.resourceUrl.scheme() == QLatin1String("kns")) {
+        return findResourceByPackageName(filter.resourceUrl);
+    } else if (filter.state >= AbstractResource::Installed) {
         QVector<AbstractResource*> ret;
         foreach(AbstractResource* r, m_resourcesByName) {
             if(r->state()>=filter.state && (r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive)))
@@ -381,7 +377,6 @@ ResultsStream * KNSBackend::findResourceByPackageName(const QUrl& search)
     const auto entryid = pathParts.at(1);
 
     auto stream = new ResultsStream(QStringLiteral("KNS-byname-")+entryid);
-
     auto start = [this, entryid, stream, providerid]() {
         m_responsePending = true;
         m_engine->fetchEntryById(entryid);
@@ -414,4 +409,9 @@ AbstractBackendUpdater* KNSBackend::backendUpdater() const
     return m_updater;
 }
 
+QString KNSBackend::displayName() const
+{
+    return QStringLiteral("KNewStuff");
+}
+
 #include "KNSBackend.moc"
