@@ -23,6 +23,7 @@
 #include <AppStreamQt/icon.h>
 #include <AppStreamQt/image.h>
 #include <AppStreamQt/release.h>
+#include <appstream/AppStreamUtils.h>
 #include <KLocalizedString>
 #include <KToolInvocation>
 #include <QIcon>
@@ -128,57 +129,13 @@ bool AppPackageKitResource::isTechnical() const
     return false;
 }
 
-static QUrl imageOfKind(const QList<AppStream::Image>& images, AppStream::Image::Kind kind)
-{
-    QUrl ret;
-    Q_FOREACH (const AppStream::Image &i, images) {
-        if (i.kind() == kind) {
-            ret = i.url();
-            break;
-        }
-    }
-    return ret;
-}
-
-static QUrl screenshot(const AppStream::Component& comp, AppStream::Image::Kind kind)
-{
-    QUrl ret;
-    Q_FOREACH (const AppStream::Screenshot &s, comp.screenshots()) {
-        ret = imageOfKind(s.images(), kind);
-        if (s.isDefault() && !ret.isEmpty())
-            break;
-    }
-    return ret;
-}
-
-QUrl AppPackageKitResource::screenshotUrl()
-{
-    return screenshot(m_appdata, AppStream::Image::KindSource);
-}
-
-QUrl AppPackageKitResource::thumbnailUrl()
-{
-    return screenshot(m_appdata, AppStream::Image::KindThumbnail);
-}
-
-void AppPackageKitResource::invokeApplication() const
-{
-    const QStringList exes = m_appdata.provided(AppStream::Provided::KindBinary).items();
-    if (exes.isEmpty()) {
-        const auto servicePath = QStandardPaths::locate(QStandardPaths::ApplicationsLocation, m_appdata.id());
-        QProcess::startDetached(QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/discover/runservice"), {servicePath});
-    } else {
-        QProcess::startDetached(exes.constFirst());
-    }
-}
-
 void AppPackageKitResource::fetchScreenshots()
 {
     QList<QUrl> thumbnails, screenshots;
 
     Q_FOREACH (const AppStream::Screenshot &s, m_appdata.screenshots()) {
-        const QUrl thumbnail = imageOfKind(s.images(), AppStream::Image::KindThumbnail);
-        const QUrl plain = imageOfKind(s.images(), AppStream::Image::KindSource);
+        const QUrl thumbnail = AppStreamUtils::imageOfKind(s.images(), AppStream::Image::KindThumbnail);
+        const QUrl plain = AppStreamUtils::imageOfKind(s.images(), AppStream::Image::KindSource);
         if (plain.isEmpty())
             qWarning() << "invalid screenshot for" << name();
 
@@ -220,3 +177,15 @@ void AppPackageKitResource::fetchChangelog()
     }
     emit changelogFetched(changelog);
 }
+
+void AppPackageKitResource::invokeApplication() const
+{
+    const QStringList exes = m_appdata.provided(AppStream::Provided::KindBinary).items();
+    if (exes.isEmpty()) {
+        const auto servicePath = QStandardPaths::locate(QStandardPaths::ApplicationsLocation, m_appdata.id());
+        QProcess::startDetached(QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/discover/runservice"), {servicePath});
+    } else {
+        QProcess::startDetached(exes.constFirst());
+    }
+}
+
