@@ -30,11 +30,15 @@
 #include <KActionCollection>
 #include "Transaction/TransactionModel.h"
 #include "Category/CategoryModel.h"
+#include "utils.h"
 #include <QDebug>
 #include <QCoreApplication>
 #include <QThread>
 #include <QAction>
 #include <QMetaProperty>
+#include <KLocalizedString>
+#include <KSharedConfig>
+#include <KConfigGroup>
 
 ResourcesModel *ResourcesModel::s_self = nullptr;
 
@@ -49,9 +53,11 @@ ResourcesModel::ResourcesModel(QObject* parent, bool load)
     : QObject(parent)
     , m_initializingBackends(0)
     , m_actionCollection(nullptr)
+    , m_currentApplicationBackend(nullptr)
 {
     init(load);
     connect(this, &ResourcesModel::allInitialized, this, &ResourcesModel::fetchingChanged);
+    connect(this, &ResourcesModel::backendsChanged, this, &ResourcesModel::initApplicationsBackend);
 }
 
 void ResourcesModel::init(bool load)
@@ -61,6 +67,18 @@ void ResourcesModel::init(bool load)
 
     if(load)
         QMetaObject::invokeMethod(this, "registerAllBackends", Qt::QueuedConnection);
+
+
+    QAction* updateAction = new QAction(this);
+    updateAction->setIcon(QIcon::fromTheme(QStringLiteral("system-software-update")));
+    updateAction->setText(i18nc("@action Checks the Internet for updates", "Check for Updates"));
+    updateAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
+    connect(this, &ResourcesModel::fetchingChanged, updateAction, [updateAction, this](){
+        updateAction->setEnabled(!isFetching());
+    });
+    connect(updateAction, &QAction::triggered, this, &ResourcesModel::checkForUpdates);
+
+    m_ownActions += updateAction;
 }
 
 ResourcesModel::ResourcesModel(const QString& backendName, QObject* parent)
@@ -118,6 +136,7 @@ void ResourcesModel::callerFetchingChanged()
         int idx = m_backends.indexOf(backend);
         Q_ASSERT(idx>=0);
         m_backends.removeAt(idx);
+//         Q_EMIT backendsChanged();
         CategoryModel::global()->blacklistPlugin(backend->name());
         backend->deleteLater();
         return;
@@ -160,17 +179,17 @@ int ResourcesModel::updatesCount() const
 
 void ResourcesModel::installApplication(AbstractResource* app)
 {
-    app->backend()->installApplication(app);
+    TransactionModel::global()->addTransaction(app->backend()->installApplication(app));
 }
 
 void ResourcesModel::installApplication(AbstractResource* app, const AddonList& addons)
 {
-    app->backend()->installApplication(app, addons);
+    TransactionModel::global()->addTransaction(app->backend()->installApplication(app, addons));
 }
 
 void ResourcesModel::removeApplication(AbstractResource* app)
 {
-    app->backend()->removeApplication(app);
+    TransactionModel::global()->addTransaction(app->backend()->removeApplication(app));
 }
 
 void ResourcesModel::registerAllBackends()
@@ -223,7 +242,7 @@ bool ResourcesModel::isFetching() const
 
 QList<QAction*> ResourcesModel::messageActions() const
 {
-    QList<QAction*> ret;
+    QList<QAction*> ret = m_ownActions;
     foreach(AbstractResourcesBackend* b, m_backends) {
         ret += b->messageActions();
     }
@@ -253,7 +272,7 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
     Q_ASSERT(!streams.contains(nullptr));
     if (streams.isEmpty()) {
         qWarning() << "no streams to aggregate!!";
-        destruction(nullptr);
+        QTimer::singleShot(0, this, &AggregatedResultsStream::clear);
     }
 
     for (auto stream: streams) {
@@ -268,9 +287,9 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
 
 void AggregatedResultsStream::addResults(const QVector<AbstractResource *>& res)
 {
-     m_results += res;
+    m_results += res;
 
-     m_delayedEmission.start();
+    m_delayedEmission.start();
 }
 
 void AggregatedResultsStream::emitResults()
@@ -286,6 +305,11 @@ void AggregatedResultsStream::emitResults()
 void AggregatedResultsStream::destruction(QObject* obj)
 {
     m_streams.remove(obj);
+    clear();
+}
+
+void AggregatedResultsStream::clear()
+{
     if (m_streams.isEmpty()) {
         emitResults();
         Q_EMIT finished();
@@ -321,3 +345,55 @@ AbstractResource* ResourcesModel::resourceForFile(const QUrl& file)
     }
     return ret;
 }
+
+void ResourcesModel::checkForUpdates()
+{
+    for(auto backend: qAsConst(m_backends))
+        backend->checkForUpdates();
+}
+
+QVector<AbstractResourcesBackend *> ResourcesModel::applicationBackends() const
+{
+    return kFilter<QVector<AbstractResourcesBackend*>>(m_backends, [](AbstractResourcesBackend* b){ return b->hasApplications(); });
+}
+
+QVariantList ResourcesModel::applicationBackendsVariant() const
+{
+    return kTransform<QVariantList>(applicationBackends(), [](AbstractResourcesBackend* b) {return QVariant::fromValue<QObject*>(b);});
+}
+
+AbstractResourcesBackend* ResourcesModel::currentApplicationBackend() const
+{
+    return m_currentApplicationBackend;
+}
+
+void ResourcesModel::setCurrentApplicationBackend(AbstractResourcesBackend* backend, bool write)
+{
+    if (backend != m_currentApplicationBackend) {
+        if (write) {
+            KConfigGroup settings(KSharedConfig::openConfig(), "ResourcesModel");
+            if (backend)
+                settings.writeEntry("currentApplicationBackend", backend->name());
+            else
+                settings.deleteEntry("currentApplicationBackend");
+        }
+
+        qDebug() << "setting currentApplicationBackend" << backend;
+        m_currentApplicationBackend = backend;
+        Q_EMIT currentApplicationBackendChanged(backend);
+    }
+}
+
+void ResourcesModel::initApplicationsBackend()
+{
+    KConfigGroup settings(KSharedConfig::openConfig(), "ResourcesModel");
+    const QString name = settings.readEntry<QString>("currentApplicationBackend", QStringLiteral("packagekit-backend"));
+
+    const auto backends = applicationBackends();
+    auto idx = kIndexOf(backends, [name](AbstractResourcesBackend* b) { return b->name() == name; });
+    if (idx<0) {
+        idx = kIndexOf(backends, [](AbstractResourcesBackend* b) { return b->hasApplications(); });
+        qDebug() << "falling back applications backend to" << idx;
+    }
+    setCurrentApplicationBackend(backends.value(idx, nullptr), false);
+}
